//
// ArfMaker.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Runs arfgen to produce an ARF
//
#include "string.h"
#include "Tracer.h"
#include "metatask.h"
#include "errstr.h"
#include "param.h"
#include "stringutil.h"

#include "arfgen.h"   // tasks

#include "ArfMaker.h"

ArfMaker::ArfMaker(string spectrumset, string arfset,
              bool withbadpixcorr, bool useodfatt,
              string badpixlocation, bool withrmfset,
              string rmfset, bool extendedsource, 
              bool withdetbins, unsigned xbins, unsigned ybins, 
              bool withsourcepos, string sourcecoords, 
              float sourcex, float sourcey,
              bool keeparf)
{
  PROC;

  /* Set up the parameters for calling arfgen */
    openParameters("arfgen");

  /* First the input parameters */
    setParameter("spectrumset",spectrumset);
    setParameter("arfset",arfset);
    setParameter("withbadpixcorr",withbadpixcorr);
    setParameter("useodfatt",useodfatt);

    if (withrmfset) { // To avoid removing arguments
       errstr<<"Not using the RMF "<<rmfset<<" for ARF creation"
             << Sas::message(Msg::App, Msg::Noisy);
    }
    setParameter("withrmfset",withrmfset);
    setParameter("rmfset",rmfset);       
    setParameter("extendedsource",extendedsource); 
    setParameter("withdetbins",withdetbins); 
    setParameter("detxbins",(int)xbins); 
    setParameter("detybins",(int)ybins); 
    setParameter("withsourcepos",withsourcepos);
    setParameter("sourcecoords",sourcecoords);
    setParameter("sourcex",sourcex);
    setParameter("sourcey",sourcey);

   /* Set the badpixlocation filename, removing any table extension */
    std::vector<std::string> v(breakup(badpixlocation, ":"));
    std::string BadpixSet = v[0];

    errstr<<" Badpix "<<BadpixSet
          << Sas::message(Msg::App, Msg::Verbose);
    
    setParameter("badpixlocation",BadpixSet);

   /* Set the detectormap type depending on source type */
    if (extendedsource) 
    {
       setParameter("detmaptype","flat");
    }
    else{
       setParameter("detmaptype","psf");
    }

  /* Set the other parameters */
    setParameter("withdetbounds",false);
    setParameter("modeleffarea",true);
    setParameter("modelquantumeff",true);
    setParameter("modelfiltertrans",true);
    setParameter("modelee",true);
    setParameter("setbackscale",true); // Set the BACKSCAL value
    setParameter("keeparfset",keeparf); 

  /* Now run arfgen using these parameters */ 
    task(arfgen);
   
  /* Shutdown */
    closeParameters();

  ENDPROC;
}

ArfMaker::~ArfMaker()
{
  PROC;
  ENDPROC;
}

