// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: EnergySpectrum
// Author: Richard Saxton
//
// Description: Provides an interface to data consisting of a value 
//              indexed by an energy value
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: EnergySpectrum.cc,v 1.1 2013/06/06 15:57:39 rsaxton Exp $

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes
#include "EnergySpectrum.h"

#include "Dal.h"
#include "errstr.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class EnergySpectrum

// Supports data consisting of an energy, energy width and data value

// -----------------------------------------------------------------------------------

  /* Constructor from input vectors */
  EnergySpectrum::EnergySpectrum(vector<float> energy, 
                                 vector<float> energy_width, 
                                 vector<float> value):
    _energy(energy),
    _energyWidth(energy_width), _value(value)
    {
      PROC;

       // Set the number of elements
        _N = energy.size();
    }

/* Destructor */
EnergySpectrum:: ~EnergySpectrum()
{
  PROC;
  ENDPROC;
};

// Method: size() - returns the number of elements in the arrays
  int EnergySpectrum::size() const
    {
      PROC; RETURN _N;
    }

// Method: return the count rate at this energy after linear interpolation
// NB: Technical debt - this should use hermiteInterpolator
  float EnergySpectrum::value(float energy) const
    {
      PROC;

        float value=0.0;

       // Extrapolations
        if (energy < _energy[0]) {
           value = _value[0] + (_value[1]-_value[0]) * 
                       (energy - _energy[0]) / (_energy[1] - _energy[0]);           
        } else if (energy >= _energy[_N-1]) {
           value = _value[_N-1] + (_value[_N-1] -_value[_N-2]) *
                   (energy - _energy[_N-1]) / (_energy[_N-1] - _energy[_N-2]);  
       // Interpolation
        } else {
           for (int i=0; i<_N-2; i++) {
               if (energy>=_energy[i] && energy <_energy[i+1]) {
                  value = _value[i] + (_value[i+1]-_value[i]) *  
                       (energy - _energy[i]) / (_energy[i+1] - _energy[i]);
                  break;
               }
           }
        }

        return value;
     }

// Method: return the value at this array element
  float EnergySpectrum::value(int i) const
    {
        return _value[i];
    }

// Method: return the value at this array element
  float EnergySpectrum::value(unsigned i) const
    {
        return _value[i];
    }

// Method: sum() - return the sum of all values in the array
  float EnergySpectrum::sum() const
    {
        float sum=0.0;

        for (int i=0; i<_N; i++) 
            sum += _value[i];
 
        return sum;
    }

// Method: energy() - return the energy array
  vector<float> EnergySpectrum::energy() const
    {
      PROC;
        return _energy;
    }

// Method: energyWidth() - return the energy width array
  vector<float> EnergySpectrum::energyWidth() const
    {
      PROC;
        return _energyWidth;
    }



