//
// Module: Eradial
// Author: Richard Saxton 
//
// Description: Main class for calculating  a source radial profile
//
// Modifications: See ChangeLog
//

#ifndef Eradial_H
#define Eradial_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "CalServer.h"
#include "STime.h"

/* Local Classes */
#include "ImageRegion.h"
#include "CalSetter.h"


/****************************************************
 *
 * class Eradial
 *
 ***************************************************/

class Eradial
{
public:

  /* Constructor */
  Eradial();

  /* Destructor */
  ~Eradial();

  /* Set the accuracy level as specified by the user */
  void setPsfType(string & psfType) const; 

  /* Get the PSF radial profile */
  vector<double> calcPsfArray(ImageRegion::RadialProfile r, 
                         double energy, double theta, double phi);

  /* Write the output file */
  const void writerpfile(DataSet * rpset, ImageRegion::RadialProfile r,
                  vector<double> psfdata, double * bestFitPSF,
		  double * linCoeffs, double rms, 
                  DatafileAttributes * imgatts) const;

//  const string getPostFix();

private:


  /* Member fields */
  ImageRegion * srcreg;
  CalSetter * _calset;
  string  _postfix;
};

#endif

