//
// RmfMaker.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Runs rmfgen to produce an ARF
//
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>

#include <rmfgen.h>   // tasks

#include "RmfMaker.h"

RmfMaker::RmfMaker(string spectrumset, string rmfset,
		   bool extendedsource, 
                   bool withdetbins, unsigned xbins, unsigned ybins,
                   bool withsourcepos, string sourcecoords,
                   float sourcex, float sourcey,
		   bool withenergybins, float emin,float emax, int nbins)
{
  PROC;

  /* Set up the parameters for calling rmfgen */
    openParameters("rmfgen");

  /* First the input parameters */
    setParameter("spectrumset",spectrumset);
    setParameter("rmfset",rmfset);

  /* Set the other parameters */

    setParameter("extendedsource",extendedsource);
                                                                                
    if (extendedsource)
    {
       setParameter("detmaptype","flat");
    }
    else{
       setParameter("detmaptype","psf");
    }

    setParameter("withsourcepos",withsourcepos);
    setParameter("sourcecoords",sourcecoords);
    setParameter("sourcex",sourcex);
    setParameter("sourcey",sourcey);

  /* Set the energy bin defaults explicitly here if necessary */
    setParameter("withenergybins",withenergybins);    
    if (withenergybins) {
       setParameter("energymin",emin );
       setParameter("energymax",emax );
       setParameter("nenergybins",nbins);
    } else { 
       setParameter("energymin",0.05);
       setParameter("energymax",12.0);
       setParameter("nenergybins",120);
    }

  /* Now run rmfgen using these parameters */ 
    task(rmfgen);
   
  /* Shutdown */
    closeParameters();

  ENDPROC;
}

RmfMaker::~RmfMaker()
{
  PROC;
  ENDPROC;
}

