//
//	test the EnergySpectrum, FluxedSpectrum and ARF classes
//
#include <cstdlib>
#include <valarray>
#include <iostream>
#include <cassert>
#include "EnergySpectrum.h"
#include "ARF.h"
#include "FluxedSpectrum.h"

using namespace std;

namespace {
bool isClose(float a, float b)
{
    return fabs(a-b) < 1.e-5;
}
}

int main()
{
      /* 
         Test the energyspectrum class with vectors
      */
        float interim1[] = {0.1,0.5,1.0,5.0,10.0};
        float interim2[] = {0.1,0.5,1.0,5.0,10.0};
        float interim3[] = {10.0,20.0,30.0,40.0,50.0};
        vector<float> energy(5);
        vector<float> energy_width(5);
        vector<float> strength(5);
        for (unsigned n=0; n<5; ++n) {
              energy[n] = interim1[n];
              energy_width[n] = interim2[n];
              strength[n] = interim3[n];
        }

        EnergySpectrum e = EnergySpectrum(energy,
                                 energy_width,
                                 strength);

       // energy array 
        float enout = e.energy()[1];
        cout<<energy[1]<<" "<<enout<<endl;
        assert(isClose(energy[1], enout)); 

       // energy width array 
        enout = e.energyWidth()[1];
        cout<<energy_width[1]<<" "<<enout<<endl;
        assert(isClose(energy_width[1], enout)); 

       // Value at array element
        float value = e.value(1);
        cout<<strength[1]<<" "<<value<<endl;
        assert(isClose(strength[1], value)); 
        
      /* 
         Test the fluxedspectrum class 
      */
        string filename = "fluxedspec.ds";
        FluxedSpectrum f = FluxedSpectrum(filename);

        // Read the energy, width, flux and countrate arrays
        enout = f.energy()[1];
        cout<<"Fluxed energy bin [1] = "<<enout<<endl;
        enout = f.energyWidth()[1];
        cout<<"Fluxed energy bin width [1] = "<<enout<<endl;
        enout = f.value(5);
        cout<<"Count rate in energy bin [5] = "<<enout<<endl;
        enout = f.flux()[5];
        cout<<"Flux in energy bin [5] = "<<enout<<endl;
        
        // Check the interpolation 
        float e1 = f.energy()[5], e2 = f.energy()[6];
        float v1 = f.value(5), v2 = f.value(6);
        float mid_energy = (e1 + e2) / 2.0;
        float v3 = f.value(mid_energy);
        cout<<"Interpolated count rate: "<<v3<<" from "
            <<v1<<" "<<v2<<" "<<e1<<" "<<e2<<endl;

      /* 
         Test the ARF class 
      */
        string filename2 = "arf.ds";
        ARF *a = new ARF(filename2);

        // Read the energy, width, and effective area arrays
        enout = a->energy()[1];
        cout<<"ARF energy bin [1] = "<<enout<<endl;
        enout = a->energyWidth()[1];
        cout<<"ARF energy bin width [1] = "<<enout<<endl;
        enout = a->value(1);
        cout<<"Effective area in energy bin [1] = "<<enout<<endl;

        delete a; a=0;

	return 0;
}
